﻿/// <summary>
/// source: Http://www.WayneCliffordBarker.co.za
/// license: free to use by any individual or company for what ever purpose.
/// disclaimer: All content provided on this site "www.waynecliffordbarker.co.za" is for informational purposes only. Although the owner takes care to represent accurate content, the owner makes no representations as to the accuracy or completeness of any information on this site or found by following any link on this site. Neither the  owner of www.waynecliffordbarker.co.za nor any author of the site will be liable for any errors or omissions in this information nor for the availability of this information. The owner of www.waynecliffordbarker.co.za nor any author of the site will not be liable for any losses, injuries, or damages from the display or use of this information obtained from this site.
/// </summary>

using System;
using System.Collections.Generic;
using HelloWorldRouting.MyRoutingClasses.Message1;
using HelloWorldRouting.MyRoutingClasses.Message2;
using WayneCliffordBarker.RoutingEngine;
using WayneCliffordBarker.RoutingEngine.Concerete;
using WayneCliffordBarker.RoutingEngine.Interface;
using WayneCliffordBarker.RoutingEngine.Model;
using WayneCliffordBarker.RoutingEngine.Persisters;

namespace HelloWorldRouting
{
    public class RouteTest : Ilogger
    {

        public void Test1()
        {
            WriteAlert("Executing Test - Single message, Id by content, validate content, single transform, single delivery");

            // Mock Messages
            DataPacket packet1 = new DataPacket();
            packet1.Source = new Uri(@"http:\\www.waynecliffordbarker.co.za");
            packet1.Content = "123,Hello World,Wayne,Clifford,Barker";

            // Rules
            List<RoutingRule> routingRules = new List<RoutingRule>();

            RoutingRule routingRule1 = new RoutingRule();
            routingRule1.RuleId = 1;
            routingRule1.RuleDescription = "Single message, Id by content, single transform, single delivery";
            routingRule1.IdentifierInstance = new IdentifyMessage1RuleByContent("123");
            routingRule1.ValidatorInstance = new ValidateMessage1();

            RoutingDestination routingDestination1 = new RoutingDestination();
            routingDestination1.TransformerInstance = new TransformMessage1ToPipeFormat();
            routingDestination1.DeliveryMechanismInstance = new WriteFileToDisk(@"c:\tests\test1\transform1.txt");
            routingRule1.Destinations.Add(routingDestination1);

            routingRules.Add(routingRule1);

            // Execute rules
            RoutingEngine engine = new RoutingEngine(this, routingRules);
            engine.Route(packet1);            

        }

        public void Test2()
        {
            WriteAlert("Executing Test - Single message, Id by source, validate content, single transform, single delivery");

            // Mock Messages
            DataPacket packet1 = new DataPacket();
            packet1.Source = new Uri(@"http:\\www.waynecliffordbarker.co.za");
            packet1.Content = "123,Hello World,Wayne,Clifford,Barker";

            // Rules
            List<RoutingRule> routingRules = new List<RoutingRule>();

            RoutingRule routingRule1 = new RoutingRule();
            routingRule1.RuleId = 1;
            routingRule1.RuleDescription = "Single message, Id by source, validate content, single transform, single delivery";
            routingRule1.IdentifierInstance = new IdentifyBySource(@"http:\\www.waynecliffordbarker.co.za");
            routingRule1.ValidatorInstance = new ValidateMessage1();

            RoutingDestination routingDestination1 = new RoutingDestination();
            routingDestination1.TransformerInstance = new TransformMessage1ToPipeFormat();
            routingDestination1.DeliveryMechanismInstance = new WriteFileToDisk(@"c:\tests\test2\rule1_transform1.txt");
            routingRule1.Destinations.Add(routingDestination1);

            routingRules.Add(routingRule1);

            // Execute rules
            RoutingEngine engine = new RoutingEngine(this, routingRules);
            engine.Route(packet1);

        }

        public void Test3()
        {
            WriteAlert("Executing Test - Single message, Id by content, validate content, multiple transforms and multiple deliveries");

            // Mock Messages
            DataPacket packet1 = new DataPacket();
            packet1.Source = new Uri(@"http:\\www.waynecliffordbarker.co.za");
            packet1.Content = "123,Hello World,Wayne,Clifford,Barker";

            // Rules
            List<RoutingRule> routingRules = new List<RoutingRule>();

            RoutingRule routingRule1 = new RoutingRule();
            routingRule1.RuleId = 1;
            routingRule1.RuleDescription = "Single message, Id by content, validate content, multiple transforms and multiple deliveries";
            routingRule1.IdentifierInstance = new IdentifyMessage1RuleByContent("123");
            routingRule1.ValidatorInstance = new ValidateMessage1();

            RoutingDestination routingDestination1 = new RoutingDestination();
            routingDestination1.TransformerInstance = new TransformMessage1ToPipeFormat();
            routingDestination1.DeliveryMechanismInstance = new WriteFileToDisk(@"c:\tests\test3\rule1_transform1.txt");
            routingRule1.Destinations.Add(routingDestination1);

            RoutingDestination routingDestination2 = new RoutingDestination();
            routingDestination2.TransformerInstance = new TransformMessage1ToJson();
            routingDestination2.DeliveryMechanismInstance = new WriteFileToDisk(@"c:\tests\test3\rule1_transform2.txt");
            routingRule1.Destinations.Add(routingDestination2);
            routingRules.Add(routingRule1);

            routingRules.Add(routingRule1);

            // Execute rules
            RoutingEngine engine = new RoutingEngine(this, routingRules);
            engine.Route(packet1);
        }

        public void Test4()
        {
            WriteAlert("Executing Test - Single message - with an Id error");

            // Mock Messages
            DataPacket packet1 = new DataPacket();
            packet1.Source = new Uri(@"http:\\www.waynecliffordbarker.co.za");
            packet1.Content = "@#$@#$@$@adada";

            // Rules
            List<RoutingRule> routingRules = new List<RoutingRule>();

            RoutingRule routingRule1 = new RoutingRule();
            routingRule1.RuleId = 1;
            routingRule1.RuleDescription = "Single message - with an error";
            routingRule1.IdentifierInstance = new IdentifyMessage1RuleByContent("123");
            routingRule1.ValidatorInstance = new ValidateMessage1();

            RoutingDestination routingDestination1 = new RoutingDestination();
            routingDestination1.TransformerInstance = new TransformMessage1ToPipeFormat();
            routingDestination1.DeliveryMechanismInstance = new WriteFileToDisk(@"c:\tests\test4\rule1_transform1.txt");
            routingRule1.Destinations.Add(routingDestination1);

            // Execute rules
            RoutingEngine engine = new RoutingEngine(this, routingRules);
            engine.Route(packet1);
        }

        public void Test5()
        {
            WriteAlert("Executing Test - Batch (2) messages - Id by content and source, validate contents, multiple (3) transforms and multiple (3) deliveries");

            // Mock Messages
            DataPacket packet1 = new DataPacket();
            packet1.Source = null;
            packet1.Content = "123,Hello World,Wayne,Clifford,Barker";

            DataPacket packet2 = new DataPacket();
            packet2.Source = new Uri(@"http:\\www.waynecliffordbarker.co.za");
            packet2.Content = "234|Hello World|Wayne|Clifford|Barker";

            // Rules
            List<RoutingRule> routingRules = new List<RoutingRule>();

            RoutingRule routingRule1 = new RoutingRule();
            routingRule1.RuleId = 1;
            routingRule1.RuleDescription = "Batch messages - Id by content validate contents, multiple (2) transforms and multiple (2) deliveries";
            routingRule1.IdentifierInstance = new IdentifyMessage1RuleByContent("123");
            routingRule1.ValidatorInstance = new ValidateMessage1();
            RoutingDestination routingDestination1 = new RoutingDestination();
            routingDestination1.TransformerInstance = new TransformMessage1ToPipeFormat();
            routingDestination1.DeliveryMechanismInstance = new WriteFileToDisk(@"c:\tests\test5\rule1_transform1.txt");
            routingRule1.Destinations.Add(routingDestination1);
            RoutingDestination routingDestination2 = new RoutingDestination();
            routingDestination2.TransformerInstance = new TransformMessage1ToJson();
            routingDestination2.DeliveryMechanismInstance = new WriteFileToDisk(@"c:\tests\test5\rule1_transform2.txt");
            routingRule1.Destinations.Add(routingDestination2);
            routingRules.Add(routingRule1);

            RoutingRule routingRule2 = new RoutingRule();
            routingRule2.RuleId = 2;
            routingRule2.RuleDescription = "Batch messages - Id by source, validate contents, single transforms and single delivery";
            routingRule2.IdentifierInstance = new IdentifyBySource(@"http:\\www.waynecliffordbarker.co.za");
            routingRule2.ValidatorInstance = new ValidateMessage2();
            RoutingDestination routingDestination3 = new RoutingDestination();
            routingDestination3.TransformerInstance = new TransformMessage2ToJson();
            routingDestination3.DeliveryMechanismInstance = new WriteFileToDisk(@"c:\tests\test5\rule2_transform1.txt");
            routingRule2.Destinations.Add(routingDestination3);
            routingRules.Add(routingRule2);

            List<DataPacket> packets = new List<DataPacket>();
            packets.Add(packet1);
            packets.Add(packet2);

            // Execute rules
            RoutingEngine engine = new RoutingEngine(this, routingRules);
            engine.Route(packets);

        }        

        public void Test6()
        {
            WriteAlert("Executing Test - Batch (3) messages - 2 succeed and 1 failure.");

            // MockContent
            DataPacket packet1 = new DataPacket();
            packet1.Source = null;
            packet1.Content = "123,Hello World,Wayne,Clifford,Barker";

            DataPacket packet2 = new DataPacket();
            packet2.Source = new Uri(@"http:\\www.waynecliffordbarker.co.za");
            packet2.Content = "234|Hello World|Wayne|Clifford|Barker";

            DataPacket packet3 = new DataPacket();
            packet3.Source = null;
            packet3.Content = "@#$@#$@$@adada";

            // Rules
            List<RoutingRule> routingRules = new List<RoutingRule>();
            RoutingRule routingRule1 = new RoutingRule();
            routingRule1.RuleId = 1;
            routingRule1.RuleDescription = "Rule 1 - CSV file Identify by content, Transform to 2 locations";
            routingRule1.IdentifierInstance = new IdentifyMessage1RuleByContent("123");
            routingRule1.ValidatorInstance = new ValidateMessage1();
            RoutingDestination routingDestination1 = new RoutingDestination();
            routingDestination1.TransformerInstance = new TransformMessage1ToPipeFormat();
            routingDestination1.DeliveryMechanismInstance = new WriteFileToDisk(@"c:\tests\test6\rule1-transform1.txt");
            routingRule1.Destinations.Add(routingDestination1);
            RoutingDestination routingDestination2 = new RoutingDestination();
            routingDestination2.TransformerInstance = new TransformMessage1ToJson();
            routingDestination2.DeliveryMechanismInstance = new WriteFileToDisk(@"c:\tests\test6\rule1-transform2.txt");
            routingRule1.Destinations.Add(routingDestination2);
            routingRules.Add(routingRule1);

            RoutingRule routingRule2 = new RoutingRule();
            routingRule2.RuleId = 2;
            routingRule2.RuleDescription = "Rule 2 - CSV file identified by source, transform to 1 locations";
            routingRule2.IdentifierInstance = new IdentifyBySource(@"http:\\www.waynecliffordbarker.co.za");
            routingRule2.ValidatorInstance = new ValidateMessage2();
            RoutingDestination routingDestination3 = new RoutingDestination();
            routingDestination3.TransformerInstance = new TransformMessage2ToJson();
            routingDestination3.DeliveryMechanismInstance = new WriteFileToFtp(new Uri("ftp://255.255.255.255/Upload/rule2-transform1.json"), "Ftp", "sdsadsads");
            routingRule2.Destinations.Add(routingDestination3);
            routingRules.Add(routingRule2);

            List<DataPacket> packets = new List<DataPacket>();
            packets.Add(packet1);
            packets.Add(packet3);
            packets.Add(packet2);

            // Execute rules
            RoutingEngine engine = new RoutingEngine(this, routingRules);
            IEnumerable<Tuple<bool, DataPacket>> results = engine.Route(packets);

            WriteInformation("Results:");
            foreach (Tuple<bool, DataPacket> result in results)
            {
                WriteInformation(result.Item2.Content + ", processed =" + result.Item1);
            }
            WriteInformation("");
        }

        public void WriteError(string line)
        {
            Console.ForegroundColor = ConsoleColor.Red;
            Console.WriteLine(line);
        }

        public void WriteAlert(string line)
        {
            Console.ForegroundColor = ConsoleColor.DarkYellow;
            Console.WriteLine(line);
        }

        public void WriteInformation(string line)
        {
            Console.ForegroundColor = ConsoleColor.White;
            Console.WriteLine(line);
        }
    }
}
